@@ -47,23 +47,17 @@ func (cmd *helmUpdateCmd) Run(ctx context.Context) error {
 	}
 
 	if cmd.Commit {
-		g, err := git.PlainOpen(gitRootPath)
-		if err != nil {
-			return err
-		}
-		wt, err := g.Worktree()
-		if err != nil {
-			return err
-		}
-		gitStatus, err := wt.Status()
+		gitStatus, err := git2.GetWorktreeStatus(ctx, gitRootPath)
 		if err != nil {
 			return err
 		}
 		for pth, s := range gitStatus {
 			if strings.HasPrefix(pth, ".helm-charts/") {
+				status.Trace(ctx, "gitStatus=%s", gitStatus.String())
 				return fmt.Errorf("--commit can only be used when .helm-chart directory is clean")
 			}
 			if (s.Staging != git.Untracked && s.Staging != git.Unmodified) || (s.Worktree != git.Untracked && s.Worktree != git.Unmodified) {
+				status.Trace(ctx, "gitStatus=%s", gitStatus.String())
 				return fmt.Errorf("--commit can only be used when the git worktree is unmodified")
 			}
 		}
