@@ -17,10 +17,10 @@ func (cmd *listTargetsCmd) Help() string {
 }
 
 func (cmd *listTargetsCmd) Run(ctx context.Context) error {
-	return withKluctlProjectFromArgs(ctx, cmd.ProjectFlags, nil, true, false, func(ctx context.Context, p *kluctl_project.LoadedKluctlProject) error {
+	return withKluctlProjectFromArgs(ctx, cmd.ProjectFlags, nil, false, true, false, func(ctx context.Context, p *kluctl_project.LoadedKluctlProject) error {
 		var result []*types.Target
-		for _, t := range p.DynamicTargets {
-			result = append(result, t.Target)
+		for _, t := range p.Targets {
+			result = append(result, t)
 		}
 		return outputYamlResult(ctx, cmd.Output, result, false)
 	})
