@@ -2,18 +2,18 @@ package deployment
 
 import (
 	"fmt"
-	"github.com/fluxcd/pkg/kustomize"
 	"github.com/hashicorp/go-multierror"
 	"github.com/kluctl/kluctl/v2/pkg/helm"
 	"github.com/kluctl/kluctl/v2/pkg/k8s"
 	"github.com/kluctl/kluctl/v2/pkg/sops"
 	"github.com/kluctl/kluctl/v2/pkg/types"
 	"github.com/kluctl/kluctl/v2/pkg/utils"
+	"github.com/kluctl/kluctl/v2/pkg/utils/flux_utils/kustomize"
+	securefs "github.com/kluctl/kluctl/v2/pkg/utils/flux_utils/kustomize/filesys"
 	"github.com/kluctl/kluctl/v2/pkg/utils/uo"
 	"github.com/kluctl/kluctl/v2/pkg/vars"
 	"github.com/kluctl/kluctl/v2/pkg/yaml"
 	"io/fs"
-	"k8s.io/apimachinery/pkg/runtime/schema"
 	"os"
 	"path"
 	"path/filepath"
@@ -107,9 +107,8 @@ func (di *DeploymentItem) getCommonLabels() map[string]string {
 }
 
 func (di *DeploymentItem) getCommonAnnotations() map[string]string {
-	a := map[string]string{
-		"kluctl.io/deployment-item-dir": filepath.ToSlash(di.RelToSourceItemDir),
-	}
+	a := di.Project.GetCommonAnnotations()
+	a["kluctl.io/deployment-item-dir"] = filepath.ToSlash(di.RelToSourceItemDir)
 	if di.Config.SkipDeleteIfTags {
 		a["kluctl.io/skip-delete-if-tags"] = "true"
 	}
@@ -181,7 +180,16 @@ func (di *DeploymentItem) isHelmValuesYaml(p string) bool {
 
 func (di *DeploymentItem) newHelmRelease(subDir string) (*helm.Release, error) {
 	configPath := yaml.FixPathExt(filepath.Join(di.RenderedDir, subDir, "helm-chart.yaml"))
-	helmChartsDir := filepath.Join(di.Project.source.dir, ".helm-charts")
+
+	var helmChartsDir string
+	if di.Project.source == di.Project.getRootProject().source {
+		// deployment item is part of the root project repo, so it's allowed to be in a relative dir
+		helmChartsDir = filepath.Join(di.Project.source.dir, di.Project.getRootProject().relDir, ".helm-charts")
+	} else {
+		// deployment item is part of a git-included project, so it must be at the repo root
+		// TODO this limitation should be lifted in some way (search multiple pathes?)
+		helmChartsDir = filepath.Join(di.Project.source.dir, ".helm-charts")
+	}
 
 	hr, err := helm.NewRelease(di.Project.source.dir, filepath.Join(di.RelToSourceItemDir, subDir), configPath, helmChartsDir, di.ctx.HelmCredentials)
 	if err != nil {
@@ -225,6 +233,8 @@ func (di *DeploymentItem) renderHelmCharts() error {
 			}
 		}
 
+		di.Config.RenderedHelmChartConfig = hr.Config
+
 		return hr.Render(di.ctx.Ctx, di.ctx.K, di.ctx.K8sVersion, di.ctx.SopsDecrypter)
 	})
 	if err != nil {
@@ -514,18 +524,13 @@ func (di *DeploymentItem) buildKustomize() error {
 		return err
 	}
 
-	resources, _, _ := ky.GetNestedStringList("resources")
-	for _, r := range resources {
-		p := filepath.Join(di.RenderedDir, r)
-		if utils.IsFile(p) {
-			err = sops.MaybeDecryptFile(di.ctx.SopsDecrypter, p)
-			if err != nil {
-				return err
-			}
-		}
+	fs, err := securefs.MakeFsOnDiskSecureBuild(di.RenderedSourceRootDir)
+	if err != nil {
+		return err
 	}
 
-	rm, err := kustomize.SecureBuild(di.RenderedSourceRootDir, di.RenderedDir, true)
+	fs = sops.NewDecryptingFs(fs, di.ctx.SopsDecrypter)
+	rm, err := kustomize.Build(fs, di.RenderedDir)
 	if err != nil {
 		return err
 	}
@@ -538,37 +543,12 @@ func (di *DeploymentItem) buildKustomize() error {
 		}
 		o := uo.FromMap(y)
 		di.Objects = append(di.Objects, o)
+		di.Config.RenderedObjects = append(di.Config.RenderedObjects, o.GetK8sRef())
 	}
 
 	return nil
 }
 
-var crdGV = schema.GroupKind{Group: "apiextensions.k8s.io", Kind: "CustomResourceDefinition"}
-
-// postprocessCRDs will update api resources from freshly deployed CRDs
-// value even if the CRD is not deployed yet.
-func (di *DeploymentItem) postprocessCRDs() error {
-	if di.dir == nil {
-		return nil
-	}
-	if di.ctx.K == nil {
-		return nil
-	}
-
-	for _, o := range di.Objects {
-		gvk := o.GetK8sGVK()
-		if gvk.GroupKind() != crdGV {
-			continue
-		}
-
-		err := di.ctx.K.Resources.UpdateResourcesFromCRD(o)
-		if err != nil {
-			return err
-		}
-	}
-	return nil
-}
-
 func (di *DeploymentItem) postprocessObjects(images *Images) error {
 	if di.dir == nil {
 		return nil
@@ -583,7 +563,7 @@ func (di *DeploymentItem) postprocessObjects(images *Images) error {
 
 		_ = k8s.UnwrapListItems(o, true, func(o *uo.UnstructuredObject) error {
 			if di.ctx.K != nil {
-				di.ctx.K.Resources.FixNamespace(o, "default")
+				di.ctx.K.FixNamespace(o, "default")
 			}
 
 			// Set common labels/annotations
