@@ -106,11 +106,6 @@ func (hr *Release) Render(ctx context.Context, k *k8s.K8sCluster, k8sVersion str
 	return nil
 }
 
-func (hr *Release) GetDeprecatedChartDir() string {
-	dir := filepath.Dir(hr.ConfigFile)
-	return filepath.Join(dir, "charts", hr.Chart.GetChartName())
-}
-
 func (hr *Release) getPulledChart(ctx context.Context) (*PulledChart, error) {
 	if hr.Chart.IsLocalChart() {
 		version, err := hr.Chart.GetLocalChartVersion()
@@ -120,12 +115,6 @@ func (hr *Release) getPulledChart(ctx context.Context) (*PulledChart, error) {
 		return NewPulledChart(hr.Chart, version, hr.Chart.GetLocalPath(), false), nil
 	}
 
-	deprecatedPC := NewPulledChart(hr.Chart, hr.Config.ChartVersion, hr.GetDeprecatedChartDir(), false)
-	if deprecatedPC.CheckExists() {
-		status.Deprecation(ctx, "helm-charts-dir", "Your project has pre-pulled charts located next to the helm-chart.yaml, which is deprecated. "+
-			"Please run 'kluctl helm-pull' on your project and ensure that the deprecated charts are removed! Future versions of kluctl will ignore these locations.")
-		return deprecatedPC, nil
-	}
 	pc, err := hr.Chart.GetPulledChart(hr.baseChartsDir, hr.Config.ChartVersion)
 	if err != nil {
 		return nil, err
@@ -214,7 +203,10 @@ func (hr *Release) doRender(ctx context.Context, k *k8s.K8sCluster, k8sVersion s
 	client.Replace = true
 	client.ClientOnly = true
 	client.KubeVersion = kubeVersion
-	client.APIVersions = hr.getApiVersions(k)
+	client.APIVersions, err = hr.getApiVersions(k)
+	if err != nil {
+		return err
+	}
 
 	if hr.Config.SkipCRDs {
 		client.SkipCRDs = true
@@ -271,7 +263,7 @@ func (hr *Release) doRender(ctx context.Context, k *k8s.K8sCluster, k8sVersion s
 		// add the necessary namespace in the rendered resources
 		if k != nil {
 			err = k8s.UnwrapListItems(o, true, func(o *uo.UnstructuredObject) error {
-				k.Resources.FixNamespace(o, namespace)
+				k.FixNamespace(o, namespace)
 				return nil
 			})
 			if err != nil {
@@ -292,14 +284,17 @@ func (hr *Release) doRender(ctx context.Context, k *k8s.K8sCluster, k8sVersion s
 	return nil
 }
 
-func (hr *Release) getApiVersions(k *k8s.K8sCluster) chartutil.VersionSet {
+func (hr *Release) getApiVersions(k *k8s.K8sCluster) (chartutil.VersionSet, error) {
 	if k == nil {
-		return nil
+		return nil, nil
 	}
 
 	m := map[string]bool{}
 
-	gvks := k.Resources.GetFilteredGVKs(nil)
+	gvks, err := k.GetFilteredGVKs(nil)
+	if err != nil {
+		return nil, err
+	}
 	for _, gvk := range gvks {
 		gvStr := gvk.GroupVersion().String()
 		m[gvStr] = true
@@ -312,7 +307,7 @@ func (hr *Release) getApiVersions(k *k8s.K8sCluster) chartutil.VersionSet {
 		ret = append(ret, id)
 	}
 
-	return ret
+	return ret, nil
 }
 
 func (hr *Release) parseRenderedManifests(s string) ([]*uo.UnstructuredObject, error) {
